@@ -26,7 +26,8 @@ from .common.metrics import registry
 class WavefrontDirectClient(connection_handler.ConnectionHandler,
                             entities.WavefrontMetricSender,
                             entities.WavefrontHistogramSender,
-                            entities.WavefrontTracingSpanSender):
+                            entities.WavefrontTracingSpanSender,
+                            entities.WavefrontEventSender):
     """Wavefront direct ingestion client.
 
     Sends data directly to Wavefront cluster via the direct ingestion API.
@@ -36,10 +37,21 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
     WAVEFRONT_HISTOGRAM_FORMAT = 'histogram'
     WAVEFRONT_TRACING_SPAN_FORMAT = 'trace'
     WAVEFRONT_SPAN_LOG_FORMAT = 'spanLogs'
+    WAVEFRONT_EVENT_FORMAT = 'event'
+
+    REPORT_END_POINT = '/report'
+    EVENT_END_POINT = '/api/v2/event'
 
     # pylint: disable=too-many-arguments
-    def __init__(self, server, token, max_queue_size=50000, batch_size=10000,
-                 flush_interval_seconds=5, enable_internal_metrics=True):
+    # pylint: disable=too-many-statements
+
+    def __init__(self,
+                 server,
+                 token,
+                 max_queue_size=50000,
+                 batch_size=10000,
+                 flush_interval_seconds=5,
+                 enable_internal_metrics=True):
         """Construct Direct Client.
 
         @param server: Server address, Example: https://INSTANCE.wavefront.com
@@ -56,7 +68,7 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         @param flush_interval_seconds: Interval flush time, 5 secs by default
         @type flush_interval_seconds: int
         """
-        super(WavefrontDirectClient, self).__init__()
+        super().__init__()
         self.server = server
         self._token = token
         self._max_queue_size = max_queue_size
@@ -67,9 +79,13 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         self._histograms_buffer = queue.Queue(max_queue_size)
         self._tracing_spans_buffer = queue.Queue(max_queue_size)
         self._spans_log_buffer = queue.Queue(max_queue_size)
+        self._events_buffer = queue.Queue(max_queue_size)
         self._headers = {'Content-Type': 'application/octet-stream',
                          'Content-Encoding': 'gzip',
                          'Authorization': 'Bearer ' + token}
+        self._event_headers = {'Content-Type': 'application/json',
+                               'Content-Encoding': 'gzip',
+                               'Authorization': 'Bearer ' + token}
         self._closed = False
         self._schedule_lock = threading.Lock()
         self._timer = None
@@ -84,6 +100,11 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
             self._sdk_metrics_registry = registry.WavefrontSdkMetricsRegistry(
                 wf_metric_sender=None)
 
+        semver = utils.get_sem_ver(constants.WAVEFRONT_SDK_PYTHON)
+
+        def version():
+            return semver
+        self._sdk_metrics_registry.new_gauge('version', version)
         self._sdk_metrics_registry.new_gauge(
             'points.queue.size', self._metrics_buffer.qsize)
         self._sdk_metrics_registry.new_gauge(
@@ -140,6 +161,20 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         self._span_logs_report_errors = self._sdk_metrics_registry.new_counter(
             'span_logs.report.errors')
 
+        self._sdk_metrics_registry.new_gauge(
+            'events.queue.size', self._events_buffer.qsize)
+        self._sdk_metrics_registry.new_gauge(
+            'events.queue.remaining_capacity',
+            remaining_capacity_getter(self._events_buffer))
+        self._events_valid = self._sdk_metrics_registry.new_counter(
+            'events.valid')
+        self._events_invalid = self._sdk_metrics_registry.new_counter(
+            'events.invalid')
+        self._events_dropped = self._sdk_metrics_registry.new_counter(
+            'events.dropped')
+        self._events_report_errors = self._sdk_metrics_registry.new_counter(
+            'events.report.errors')
+
     def _report(self, points, data_format, entity_prefix, report_errors):
         r"""One api call sending one given string data.
 
@@ -149,11 +184,19 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         @type data_format: str
         """
         try:
-            params = {'f': data_format}
-            compressed_data = utils.gzip_compress(points.encode('utf-8'))
-            response = requests.post(self.server + '/report', params=params,
-                                     headers=self._headers,
-                                     data=compressed_data)
+            if data_format == self.WAVEFRONT_EVENT_FORMAT:
+                response = requests.post(self.server + self.EVENT_END_POINT,
+                                         params=None,
+                                         headers=self._event_headers,
+                                         data=points)
+            else:
+                params = {'f': data_format}
+                compressed_data = utils.gzip_compress(points.encode('utf-8'))
+                response = requests.post(self.server + self.REPORT_END_POINT,
+                                         params=params,
+                                         headers=self._headers,
+                                         data=compressed_data)
+
             self._sdk_metrics_registry.new_counter(
                 '{}.report.{}'.format(entity_prefix,
                                       response.status_code)).inc()
@@ -171,8 +214,12 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         @param data_format: Type of data to be sent
         @type data_format: str
         """
+        # Sending events through direct ingestion does not support batching.
+        batch_size = int(
+            data_format == self.WAVEFRONT_EVENT_FORMAT) or self._batch_size
+
         # Split data into chunks, each with the size of given batch_size
-        for batch in utils.chunks(batch_line_data, self._batch_size):
+        for batch in utils.chunks(batch_line_data, batch_size):
             # report once per batch
             try:
                 self._report('\n'.join(batch) + '\n', data_format,
@@ -231,6 +278,9 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         self._internal_flush(self._spans_log_buffer,
                              self.WAVEFRONT_SPAN_LOG_FORMAT, 'span_logs',
                              self._span_logs_report_errors)
+        self._internal_flush(self._events_buffer,
+                             self.WAVEFRONT_EVENT_FORMAT, 'events',
+                             self._events_report_errors)
 
     def close(self):
         """Flush all buffer before close the client."""
@@ -241,10 +291,8 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
                 self._timer.cancel()
         self._sdk_metrics_registry.close(timeout_secs=1)
 
-    # pylint: disable=too-many-arguments
-
     def send_metric(self, name, value, timestamp, source, tags):
-        """Send Metric Data via proxy.
+        """Send Metric Data via direct ingestion client.
 
         Wavefront Metrics Data format
         <metricName> <metricValue> [<timestamp>] source=<source> [pointTags]
@@ -287,11 +335,9 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         self._batch_report(metrics, self.WAVEFRONT_METRIC_FORMAT, 'points',
                            self._points_report_errors)
 
-    # pylint: disable=too-many-arguments
-
     def send_distribution(self, name, centroids, histogram_granularities,
                           timestamp, source, tags):
-        """Send Distribution Data via proxy.
+        """Send Distribution Data via direct ingestion client.
 
         Wavefront Histogram Data format
         {!M | !H | !D} [<timestamp>] #<count> <mean> [centroids]
@@ -330,7 +376,7 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         """Send a list of distribution immediately.
 
         Have to construct the data manually by calling
-        common.utils.metric_to_line_data()
+        common.utils.histogram_to_line_data()
 
         @param distributions: List of string spans data
         @type distributions: list[str]
@@ -342,7 +388,7 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
 
     def send_span(self, name, start_millis, duration_millis, source, trace_id,
                   span_id, parents, follows_from, tags, span_logs):
-        """Send span data via proxy.
+        """Send span data via direct ingestion client.
 
         Wavefront Tracing Span Data format
         <tracingSpanName> source=<source> [pointTags] <start_millis>
@@ -406,7 +452,7 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         Send a list of spans immediately.
 
         Have to construct the data manually by calling
-        common.utils.metric_to_line_data()
+        common.utils.tracing_span_to_line_data()
 
         @param spans: List of string spans data
         @type spans: list[str]
@@ -427,12 +473,65 @@ class WavefrontDirectClient(connection_handler.ConnectionHandler,
         self._batch_report(span_logs, self.WAVEFRONT_SPAN_LOG_FORMAT,
                            'span_logs', self._span_logs_report_errors)
 
+    def send_event(self, name, start_time, end_time, source, tags,
+                   annotations):
+        """Send Event Data via direct ingestion client.
+
+        Wavefront Event Data format
+        {"name": <Event Name>, "annotations": <Annotations>,
+         "hosts": <Host Name>,"startTime": <Start Time>,
+          "endTime": <End Time>, "tags": <Tags>}
+        Example: {"name": event_via_direct_ingestion, "annotations": {
+        "severity": "severe", "type": "backup", "details": "broker backup"},
+         "hosts": ["localhost"], "startTime": 1590678089,
+         "endTime": 1590679089, "tags": ["env:", "test"]}
+
+        @param name: Event Name
+        @type name: str
+        @param start_time: Event Start Time
+        @type start_time: long
+        @param end_time: Event End Time
+        @type end_time: long
+        @param source: Source
+        @type source: str
+        @param tags: Tags
+        @type tags: list[str]
+        @param annotations: Annotations
+        @type annotations: dict
+        """
+        try:
+            line_data = utils.event_to_json(
+                name, start_time, end_time, source, tags, annotations,
+                self._default_source)
+            self._events_valid.inc()
+        except ValueError as error:
+            self._events_invalid.inc()
+            raise error
+        try:
+            self._events_buffer.put(line_data)
+        except queue.Full as error:
+            self._events_dropped.inc()
+            raise error
+
+    def send_event_now(self, events):
+        """Send a list of events immediately.
+
+        Have to construct the data manually by calling
+        common.utils.event_to_json()
+
+        @param events: List of string events data
+        @type events: list[str]
+        """
+        self._batch_report(events, self.WAVEFRONT_EVENT_FORMAT, 'events',
+                           self._events_report_errors)
+
     def get_failure_count(self):
         """Get failure count for one connection."""
         return (self._points_report_errors.count() +
                 self._histograms_report_errors.count() +
                 self._spans_report_errors.count() +
-                self._span_logs_report_errors.count())
+                self._span_logs_report_errors.count() +
+                self._events_report_errors.count())
 
 
 def remaining_capacity_getter(buf):
